@@ -61,7 +61,7 @@ class TherCaller():
             print("WARNING: Corrupted bulk, THERIN bulk (str) contains an element which is 0.")
             return False
 
-    def __init__(self, programs_dir: str, database: str, theriak_version: str, cwd: str):
+    def __init__(self, programs_dir: str, database: str, theriak_version: str, verbose: bool = True):
         """Creates a TherCaller intance.
         For calling theriak, the theriak.ini file must be copied from the programs folder to the working directory.
 
@@ -71,22 +71,36 @@ class TherCaller():
             theriak_version (str): version of theriak, lookup in programs folder.
 
         """
-        self.cwd = Path(cwd)
 
         self.theriak_exe = Path(programs_dir) / "theriak"
         self.database = database
 
         self.theriak_version = theriak_version
 
+        self.verbose_mode = verbose
+
+    def call_theriak(self, pressure: int, temperature: int, bulk: str):
+        """Execute theriak.exe and returns the OUT as string.
+
+        Args:
+            pressure (int): Pressure in [bar].
+            temperature (int): Temperature in [deg C]
+            bulk (str): Bulk composition in the THERIN format. Elements in capital letters (e.g. SI, AL) and moles in brackets.
+
+        Returns:
+            str: Theriak output.
+        """
+        self.pressure = pressure
+        self.temperature = temperature
+
         # create theriak input for subprocess.run(), "\n" is interpreted as "enter"
         self.theriak_input = self.database + "\n" + "no\n"
 
-    def call_theriak(self, pressure: int, temperature: int, bulk: str):
         # write THERIN
         self.therin_PT = "    " + str(temperature) + "    " + str(pressure)
         self.therin_bulk = "1   " + bulk + "    *"
 
-        with open(self.cwd / "THERIN", "w") as therin_file:
+        with open("THERIN", "w") as therin_file:
             therin_file.write(self.therin_PT)
             therin_file.write("\n")
             therin_file.write(self.therin_bulk)
@@ -103,7 +117,7 @@ class TherCaller():
 
     def check_minimisation(self, theriak_output: str):
         """Checks for theriak calls with failed minimisation.
-        If and only if a minimisation fails, this will be marked in the theriak output by "**" before the activities of Em in solutions.
+        If and only if a minimisation fails, this will be marked in the theriak output by "**" before the activities of EM in solutions.
         Additionally below the activities block the "** activity test:" is listed for the failed solution phases.
 
         This function checks for a succesful minimisation.
@@ -118,16 +132,17 @@ class TherCaller():
         substring_if_minimisation_fail = "activity test"
 
         if substring_if_minimisation_fail in theriak_output:
-            print("WARNING: Detected a failed minimisation. For the following THERIN:")
-            print(self.therin_PT)
-            print(self.therin_bulk)
+            if self.verbose_mode is True:
+                print("WARNING: Minimisation possibly failed. For the following THERIN:")
+                print(self.therin_PT)
+                print(self.therin_bulk)
 
             return False
 
         elif substring_if_minimisation_fail not in theriak_output:
             return True
 
-    def read_theriak(self, theriak_output: str, pressure: int, temperature: int):
+    def read_theriak(self, theriak_output: str):
         theriak_output = theriak_output.splitlines()
 
         """
@@ -158,8 +173,9 @@ class TherCaller():
             fluids_stable = True
         # catch the ValueError, when no stable fluid phases predicted the block_fluid wont show in theriak output.
         except ValueError:
-            print("WARNING: No fluid phase stable at given P-T-X. Water-sat. conditions not fulfilled.")
-            # set a state, to disabkle adding fluids to the rock later on. Set an empty list as placeholder instead of the fluid block.
+            if self.verbose_mode:
+                print("WARNING: No fluid phase stable at given P-T-X. Water-sat. conditions not fulfilled.")
+            # set a state, to disable adding fluids to the rock later on. Set an empty list as placeholder instead of the fluid block.
             fluids_stable = False
             block_fluid = []
 
@@ -181,7 +197,14 @@ class TherCaller():
         # 8) chemical potential of components
         # ## OPTIONAL, to be discussed if necessary.
 
-        blocks = [block_bulk, block_Gsys, block_volume, block_fluid, block_elements, block_composition, block_deltaG]
+        # blocks = [block_bulk, block_Gsys, block_volume, block_fluid, block_elements, block_composition, block_deltaG]
+        blocks = {"block_bulk": block_bulk,
+                  "block_Gsys": block_Gsys,
+                  "block_volume": block_volume,
+                  "block_fluid": block_fluid,
+                  "block_elements": block_elements,
+                  "block_composition": block_composition,
+                  "block_deltaG": block_deltaG}
 
         # check for line overflow: ("elements in bulk" > 10) in "elements in stable phases:"
         # if last line in compostion block starts with an indendation, then there is line overflow in the output
@@ -199,20 +222,40 @@ class TherCaller():
         # drop last element "E" (only for testing in theriak)
         element_list = element_list[:-1]
 
-        """
-        Create Rock
-        """
-        rock = Rock(pressure=pressure, temperature=temperature)
+        return blocks, element_list, output_line_overflow, fluids_stable
+
+    def create_rock(self, blocks: dict, output_line_overflow: bool, fluids_stable: bool):
+        rock = Rock(pressure=self.pressure, temperature=self.temperature)
         rock.add_therin_to_reproduce(PT=self.therin_PT, bulk=self.therin_bulk)
-        rock.add_bulk_rock_composition(block_bulk=blocks[0])
-        rock.add_g_system(block_Gsys=blocks[1])
-        rock.add_minerals(block_volume=blocks[2], block_composition=blocks[5], block_elements=blocks[4],
+        rock.add_bulk_rock_composition(block_bulk=blocks["block_bulk"])
+        rock.add_g_system(block_Gsys=blocks["block_Gsys"])
+        rock.add_minerals(block_volume=blocks["block_volume"], block_composition=blocks["block_composition"], block_elements=blocks["block_elements"],
                           output_line_overflow=output_line_overflow)
         if fluids_stable:
-            rock.add_fluids(block_fluid=blocks[3], block_composition=blocks[5], block_elements=blocks[4], output_line_overflow=output_line_overflow)
-        rock.add_deltaG(block_deltaG=blocks[6])
+            rock.add_fluids(block_fluid=blocks["block_fluid"], block_composition=blocks["block_composition"], block_elements=blocks["block_elements"],
+                            output_line_overflow=output_line_overflow)
+        rock.add_deltaG(block_deltaG=blocks["block_deltaG"])
         rock.add_g_system_per_mol()
-        return rock, theriak_output, blocks, element_list
+
+        return rock
+
+    def minimisation(self, pressure: int, temperature: int, bulk: str, return_failed_minimisation: bool = False):
+        # A compositon of call_theriak --> check_minimisation --> read_theriak; returning a rock, ele_list
+        theriak_output = TherCaller.call_theriak(self, pressure=pressure, temperature=temperature, bulk=bulk)
+        minimisation_state = TherCaller.check_minimisation(self, theriak_output=theriak_output)
+        if return_failed_minimisation:
+            # overwrite minimisation_state
+            minimisation_state = True
+
+        if minimisation_state:
+            blocks, element_list, output_line_overflow, fluids_stable = TherCaller.read_theriak(self, theriak_output=theriak_output)
+            rock = TherCaller.create_rock(self, blocks=blocks, output_line_overflow=output_line_overflow, fluids_stable=fluids_stable)
+
+            return rock, element_list
+
+        else:
+            # for a failed minimisation return the raw theriak output.
+            return theriak_output
 
 
 class Rock:
@@ -281,7 +324,7 @@ class Rock:
         self.g_system = None
         self.pressure = pressure
         self.temperature = temperature
-        self.bulk_composition_n = None
+        self.bulk_composition_moles = None
         self.bulk_composition_mol_percent = None
 
     def add_therin_to_reproduce(self, PT: str, bulk: str):
@@ -308,7 +351,7 @@ class Rock:
         # convert to str --> float
         bulk_mol_percent_list = [float(n) for n in bulk_mol_percent_list]
 
-        self.bulk_composition_n = bulk_mol_list
+        self.bulk_composition_moles = bulk_mol_list
         self.bulk_composition_mol_percent = bulk_mol_percent_list
 
     def add_g_system(self, block_Gsys: list):
@@ -327,7 +370,7 @@ class Rock:
         self.g_system = g_system
 
     def add_g_system_per_mol(self):
-        composition_list = self.bulk_composition_n
+        composition_list = self.bulk_composition_moles
         sum_mol_input_elements = np.sum(composition_list)
         # divide the g_system by the molar sum of all inputed elements;
         # doing this makes the g_sys independent from the absolute input used (depends on personal format choice)
@@ -374,15 +417,7 @@ class Rock:
         idx_of_firstZEROS = [i.index("0.00000E+00") for i in metastable_list]
         metastable_list_names = [i[2] for i in metastable_list]
         metastable_list_deltaG = [i[idx + 1] for i, idx in zip(metastable_list, idx_of_firstZEROS)]
-        # try:
-        #     metastable_list_deltaG = [i[1].split()[0] for i in metastable_list]
-        #     print("!!!! METASTABLE LIST correct !!!")
-        #     print(metastable_list)
-        # except IndexError:
-        #     print("!!!! METASTABLE LIST FAIL !!!")
-        #     print(metastable_list)
-        #     print("!!!! block_deltaG !!!")
-        #     print(block_deltaG)
+
         metastable_list = np.array([metastable_list_names, metastable_list_deltaG])
         metastable_list = np.transpose(metastable_list).tolist()
         self.mineral_delta_G = metastable_list
@@ -415,7 +450,7 @@ class Phase:
         # convert composition to float
         phase_composition = [float(x) for x in phase_composition]
 
-        self.composition = phase_composition
+        self.composition_apfu = phase_composition
         # important to also save Oxygen, to calculate Fe3
 
     def add_composition_moles(self, block_elements: list, temp_name: str, output_line_overflow: bool):
